<!doctype html>
<html lang="en">
  <head>
    <!-- Required meta tags -->
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">

    <!-- Bootstrap CSS -->
    <link rel="stylesheet" href="/static/css/bootstrap.min.css">
    <link rel="stylesheet" href="/static/css/sb-admin-2.min.css">
    <link rel="stylesheet" href="/static/css/site.css">
    <script src=" /static/js/jquery-1.12.3.min.js"></script>

    <title>Scrapydd</title>
    {% block header %}
    {% end %}
  </head>
  <body>
    {% include 'topnav.html' %}
    <div id="wrapper">
    <div class="container">
    {% block content %}

    {% end %}
      </div>
      </div>
    <!-- Optional JavaScript -->
    <!-- jQuery first, then Popper.js, then Bootstrap JS -->

    <script src="/static/js/bootstrap.bundle.js"></script>
  </body>
</html>