{% extends '../base_sidebar.html' %}
{% block sidebar %}
  {% include 'sidebar_profile.html' %}
{% end sidebar %}
{% block content %}
    <div class="alert alert-success" role="alert">
      Password changed.
    </div>
{% end %}