<nav class="col-md-2 d-none d-md-block bg-light sidebar">
  <div class="sidebar-sticky">
    <ul class="nav flex-column">
      <li class="nav-item">
        <a class="nav-link active" href="/admin">
          <span data-feather="home"></span>
          Admin Panel <span class="sr-only">(current)</span>
        </a>
      </li>
      <li class="nav-item">
        <a class="nav-link" href="/admin/nodes">
          <span data-feather="file"></span>
          Nodes
        </a>
      </li>
    </ul>
  </div>
</nav>