{% extends '../base_sidebar.html' %}
{% block sidebar %}
{% include 'admin_sidebar.html' %}
{% end %}
{% block content %}
<div class="d-sm-flex align-items-center justify-content-between mb-4">
  <h1 class="h3 mb-0 text-gray-800">Dashboard</h1>
</div>
<div class="row">
  <div class="col-3">
    <div class="card border-left-primary shadow h-100 py-2">
      <div class="card-body">
        <div class="row no-gutters align-items-center">
          <div class="col mr-2">
            <div class="text-xs font-weight-bold text-primary text-uppercase mb-1">Users</div>
            <div class="h5 mb-0 font-weight-bold text-gray-800">{{ user_count }}</div>
          </div>
          <div class="col-auto">
            <i class="fas fa-calendar fa-2x text-gray-300"></i>
          </div>
        </div>
      </div>
    </div>
  </div>
  <div class="col-3">
    <div class="card border-left-primary shadow h-100 py-2">
      <div class="card-body">
        <div class="row no-gutters align-items-center">
          <div class="col mr-2">
            <div class="text-xs font-weight-bold text-primary text-uppercase mb-1">Nodes</div>
            <div class="h5 mb-0 font-weight-bold text-gray-800"> <a href="/admin/nodes">{{ active_node_count }}</a> / <a href="/admin/nodes">{{ all_node_count }}</a></div>
          </div>
          <div class="col-auto">
            <i class="fas fa-calendar fa-2x text-gray-300"></i>
          </div>
        </div>
      </div>
    </div>
  </div>
</div>
{% end %}