{% extends '../base_sidebar.html' %}
{% block sidebar %}
  {% include 'sidebar_profile.html' %}
{% end sidebar %}
{% block content %}
  <form method="POST">
    {% module xsrf_form_html() %}
  <div class="form-group row">
    <label for="oldPassword" class="col-sm-2 col-form-label">Old Password</label>
    <div class="col-sm-10">
      <input type="password" class="form-control" id="oldPassword" name="oldpassword" placeholder="Enter Old Password">
    </div>
  </div>
  <div class="form-group row">
    <label for="inputPassword" class="col-sm-2 col-form-label">New Password</label>
    <div class="col-sm-10">
      <input type="password" class="form-control" id="inputPassword" name="newpassword" placeholder="Enter New Password">
    </div>
  </div>
    <div class="form-group row">
    <label for="inputPassword2" class="col-sm-2 col-form-label">Confirm New Password</label>
    <div class="col-sm-10">
      <input type="password" class="form-control" id="inputPassword2" name="newpassword2" placeholder="Confirm New Password">
    </div>
  </div>
    {% if globals().get('error_message', False) %}
      <div class="alert alert-danger" role="alert">
      {{ error_message}}
      </div>
    {% end %}
    <div>
      <button type="summit" class="btn btn-primary btn-lg">Summit</button>
    </div>
</form>
{% end %}