<nav class="col-md-2 d-none d-md-block bg-light sidebar">
  <div class="sidebar-sticky">
    <ul class="nav flex-column">
      <li class="nav-item">
        <a class="nav-link active" href="#">
          <span data-feather="home"></span>
          Profile <span class="sr-only">(current)</span>
        </a>
      </li>
      <li class="nav-item">
        <a class="nav-link" href="/profile/change_password">
          <span data-feather="file"></span>
          Change Password
        </a>
      </li>
      <li class="nav-item">
        <a class="nav-link" href="/profile/keys">
          <span data-feather="file"></span>
          Keys
        </a>
      </li>
    </ul>
  </div>
</nav>