{% extends "base.html" %}
{% block content %}
    <form method="POST">
        {% module xsrf_form_html() %}
    <div>
        <label>Project</label>
        <input value="{{spider.project.name}}" />
    </div>
    <div>
        <label>Spider</label>
        <input value="{{spider.name}}" />
    </div>
    <div>
        <label>Cron</label>
        <input name="cron" />
    </div>
        <div><input type="submit" value="保存" /></div>
        </form>
    {% if errormsg %}
        <div>
            <font style="color:red">{{errormsg}}</font>
        </div>
    {% end %}
{% end %}